import { Pipe, PipeTransform } from '@angular/core';

interface RequirementSymbolObject {
    symbol: RequirementSymbol;
    pattern: RegExp;
}

interface RequirementSymbol {
    type: 'text' | 'icon';
    symbol: string;
    tooltip: string;
}

@Pipe({ name: 'requirementSymbols' })
export class RequirementSymbolsPipe implements PipeTransform {
    transform(lines: string[]): RequirementSymbol[] {
        const singleLine = lines.join(' ').trim();
        const matches = singleLine.match(/^((IP)|(IN-P)|(PL)|(R)|(<>)|\+|\-|\*)+/g);
        const symbols: RequirementSymbolObject[] = [
            { symbol: { type: 'text', symbol: 'IP', tooltip: 'Requirement uses in-progress credit/courses' }, pattern: /IP/ },
            { symbol: { type: 'text', symbol: 'IN-P', tooltip: 'Sub-requirement uses in progress credit/courses' }, pattern: /IN-P/ },
            { symbol: { type: 'text', symbol: 'PL', tooltip: 'Requirement/sub-requirement uses planned course' }, pattern: /PL/ },
            { symbol: { type: 'text', symbol: 'R', tooltip: 'Required sub-requirement (mandatory)' }, pattern: /R/ },
            { symbol: { type: 'text', symbol: '<>', tooltip: 'Optional/other requirement in OR\'d set complete' }, pattern: /<>/ },
            { symbol: { type: 'text', symbol: '*', tooltip: 'Optional sub-requirement, courses assigned' }, pattern: /\*}/ },
            { symbol: { type: 'icon', symbol: 'check', tooltip: 'Sub-requirement complete' }, pattern: /\+/ },
            { symbol: { type: 'icon', symbol: 'close', tooltip: 'Sub-requirement not complete' }, pattern: /\-/ },
        ];

        if (matches && matches.length > 0) {
            const symbolString = singleLine.substr(0, matches[0].length);
            return symbols.reduce((acc, symbol) => {
                return symbolString.match(symbol.pattern) !== null ? [...acc, symbol.symbol] : acc;
            }, []);
        }

        return [];
    }
}
